@@ -1,6 +1,8 @@
 /***************************************************************************
- * Copyright (C) 2015-2016 by Savoir-faire Linux                           *
+ * Copyright (C) 2015-2017 by Savoir-faire Linux                           *
  * Author: Edric Ladent Milaret <edric.ladent-milaret@savoirfairelinux.com>*
+ * Author: Anthony Léonard <anthony.leonard@savoirfairelinux.com>          *
+ * Author: Olivier Soldano <olivier.soldano@savoirfairelinux.com>          *
  *                                                                         *
  * This program is free software; you can redistribute it and/or modify    *
  * it under the terms of the GNU General Public License as published by    *
@@ -31,7 +33,9 @@
 #undef ERROR
 #undef interface
 
+//LRC
 #include "audio/settings.h"
+#include "accountmodel.h"
 #include "personmodel.h"
 #include "person.h"
 #include "fallbackpersoncollection.h"
@@ -44,22 +48,25 @@
 #include "contactmethod.h"
 #include "globalinstances.h"
 #include <availableaccountmodel.h>
+#include "pendingcontactrequestmodel.h"
+#include "profilemodel.h"
+#include "profile.h"
+#include "peerprofilecollection.h"
+#include "localprofilecollection.h"
+#include "callmodel.h"
 
+//Client
 #include "wizarddialog.h"
 #include "windowscontactbackend.h"
 #include "contactpicker.h"
 #include "contactmethodpicker.h"
 #include "globalsystemtray.h"
-#include "historydelegate.h"
-#include "contactdelegate.h"
 #include "smartlistdelegate.h"
 #include "imdelegate.h"
 #include "pixbufmanipulator.h"
 #include "settingskey.h"
+#include "contactrequestitemdelegate.h"
 
-#include "profilemodel.h"
-#include "peerprofilecollection.h"
-#include "localprofilecollection.h"
 
 CallWidget::CallWidget(QWidget* parent) :
     NavWidget(parent),
@@ -85,8 +92,6 @@ CallWidget::CallWidget(QWidget* parent) :
 
     ui->qrLabel->hide();
 
-    GlobalInstances::setPixmapManipulator(std::unique_ptr<Interfaces::PixbufManipulator>(new Interfaces::PixbufManipulator()));
-
     try {
         callModel_ = &CallModel::instance();
 
@@ -95,76 +100,22 @@ CallWidget::CallWidget(QWidget* parent) :
         connect(callModel_, SIGNAL(callStateChanged(Call*, Call::State)),
                 this, SLOT(callStateChanged(Call*, Call::State)));
 
-        connect(&AccountModel::instance()
-                , SIGNAL(dataChanged(QModelIndex,QModelIndex,QVector<int>))
-                , this
-                , SLOT(findRingAccount(QModelIndex, QModelIndex, QVector<int>)));
-
         RecentModel::instance().peopleProxy()->setFilterRole(static_cast<int>(Ring::Role::Name));
         RecentModel::instance().peopleProxy()->setFilterCaseSensitivity(Qt::CaseInsensitive);
         ui->smartList->setModel(RecentModel::instance().peopleProxy());
 
-        smartListDelegate_ = new SmartListDelegate();
-        ui->smartList->setSmartListItemDelegate(smartListDelegate_);
-
         PersonModel::instance().addCollection<PeerProfileCollection>(LoadOptions::FORCE_ENABLED);
         ProfileModel::instance().addCollection<LocalProfileCollection>(LoadOptions::FORCE_ENABLED);
 
         PersonModel::instance().
                 addCollection<WindowsContactBackend>(LoadOptions::FORCE_ENABLED);
 
-        CategorizedContactModel::instance().setSortAlphabetical(false);
-        CategorizedContactModel::instance().setUnreachableHidden(true);
-        ui->contactView->setModel(&CategorizedContactModel::instance());
-        contactDelegate_ = new ContactDelegate();
-        ui->contactView->setItemDelegate(contactDelegate_);
-
-        CategorizedHistoryModel::instance().
-                addCollection<LocalHistoryCollection>(LoadOptions::FORCE_ENABLED);
-
-        ui->historyList->setModel(CategorizedHistoryModel::SortedProxy::instance().model());
-        CategorizedHistoryModel::SortedProxy::instance().model()->sort(0, Qt::DescendingOrder);
-        ui->historyList->setHeaderHidden(true);
-        historyDelegate_ = new HistoryDelegate();
-        ui->historyList->setItemDelegate(historyDelegate_);
-
-        connect(CategorizedHistoryModel::SortedProxy::instance().model(), &QSortFilterProxyModel::layoutChanged, [=]() {
-            auto idx = CategorizedHistoryModel::SortedProxy::instance().model()->index(0,0);
-            if (idx.isValid())
-                ui->historyList->setExpanded(idx, true);
-        });
-
         connect(ui->smartList, &QTreeView::entered, this, &CallWidget::on_entered);
 
         smartListDelegate_ = new SmartListDelegate();
         ui->smartList->setSmartListItemDelegate(smartListDelegate_);
 
-        ui->historyList->setContextMenuPolicy(Qt::CustomContextMenu);
-        connect(ui->historyList, &QListView::customContextMenuRequested, [=](const QPoint& pos){
-            if (ui->historyList->currentIndex().parent().isValid()) {
-                QPoint globalPos = ui->historyList->mapToGlobal(pos);
-                QMenu menu;
-
-                ContactMethod* contactMethod = ui->historyList->currentIndex()
-                        .data(static_cast<int>(Call::Role::ContactMethod)).value<ContactMethod*>();
-
-                auto copyAction = new QAction(tr("Copy number"), this);
-                menu.addAction(copyAction);
-                connect(copyAction, &QAction::triggered, [=]() {
-                    QApplication::clipboard()->setText(contactMethod->uri());
-                });
-                if (not contactMethod->contact() || contactMethod->contact()->isPlaceHolder()) {
-                    auto addExisting = new QAction(tr("Add to contact"), this);
-                    menu.addAction(addExisting);
-                    connect(addExisting, &QAction::triggered, [=]() {
-                        ContactPicker contactPicker(contactMethod);
-                        contactPicker.move(globalPos.x(), globalPos.y() - (contactPicker.height()/2));
-                        contactPicker.exec();
-                    });
-                }
-                menu.exec(globalPos);
-            }
-        });
+        ui->contactRequestList->setItemDelegate(new ContactRequestItemDelegate());
 
         findRingAccount();
         setupOutOfCallIM();
@@ -184,12 +135,48 @@ CallWidget::CallWidget(QWidget* parent) :
                         auto idx = selected.indexes().first();
                         auto realIdx = RecentModel::instance().peopleProxy()->mapFromSource(idx);
                         ui->smartList->selectionModel()->setCurrentIndex(realIdx, QItemSelectionModel::ClearAndSelect);
-                    } else
+                    } else {
+                        RecentModel::instance().selectionModel()->clearCurrentIndex();
                         ui->smartList->clearSelection();
+                        ui->smartList->selectionModel()->clearCurrentIndex();
+                    }
                 });
 
-        connect(&NameDirectory::instance(), SIGNAL(registeredNameFound(const Account*,NameDirectory::LookupStatus,const QString&,const QString&)),
-                this, SLOT(contactLineEdit_registeredNameFound(const Account*,NameDirectory::LookupStatus,const QString&,const QString&)));
+        connect(&NameDirectory::instance(), SIGNAL(registeredNameFound(Account*,NameDirectory::LookupStatus,const QString&,const QString&)),
+                this, SLOT(contactLineEdit_registeredNameFound(Account*,NameDirectory::LookupStatus,const QString&,const QString&)));
+
+        connect(&AccountModel::instance(), SIGNAL(dataChanged(QModelIndex,QModelIndex,QVector<int>)),
+                ui->currentAccountWidget, SLOT(update()));
+
+        connect(ui->searchBtn, SIGNAL(clicked(bool)), this, SLOT(searchBtnClicked()));
+
+        connect(ui->sendContactRequestWidget, &SendContactRequestWidget::sendCRclicked, [=]{slidePage(ui->messagingPage);});
+
+        connect(ui->contactRequestWidget, &ContactRequestWidget::choiceMade, [this]() {
+            if (getSelectedAccount()->pendingContactRequestModel()->rowCount() == 0)
+                ui->mainTabMenu->setCurrentWidget(ui->mainTabMenuPage1);
+            else
+                ui->contactRequestList->selectionModel()->clear();
+        });
+
+        connect(AvailableAccountModel::instance().selectionModel(), &QItemSelectionModel::currentChanged,
+                this, &CallWidget::selectedAccountChanged);
+
+        // It needs to be called manually once to initialize the ui with the account selected at start.
+        // The second argument (previous) is set to an invalid QModelIndex as it is the first selection.
+        selectedAccountChanged(AvailableAccountModel::instance().selectionModel()->currentIndex(), QModelIndex());
+
+        // This connect() is used to initialise and track changes of profile's picture
+        connect(&ProfileModel::instance(), &ProfileModel::dataChanged,
+                ui->currentAccountWidget, &CurrentAccountWidget::setPhoto);
+
+        connect(ui->videoWidget, &VideoView::videoSettingsClicked, this, &CallWidget::settingsButtonClicked);
+        connect(ui->videoWidget, &VideoView::videoBackClicked, [=]{setActualCall(nullptr);
+                                                                    backToWelcomePage();});
+        // setup searchingfor mini spinner
+        miniSpinner_ = new QMovie(":/images/waiting.gif");
+        ui->spinnerLabel->setMovie(miniSpinner_);
+        ui->spinnerLabel->hide();
 
     } catch (const std::exception& e) {
         qDebug() << "INIT ERROR" << e.what();
@@ -200,7 +187,6 @@ CallWidget::~CallWidget()
 {
     delete ui;
     delete menu_;
-    delete contactDelegate_;
     delete imDelegate_;
     delete pageAnim_;
     delete smartListDelegate_;
@@ -247,7 +233,8 @@ CallWidget::setupOutOfCallIM()
 }
 
 void
-CallWidget::onIncomingMessage(::Media::TextRecording* t, ContactMethod* cm) {
+CallWidget::onIncomingMessage(::Media::TextRecording* t, ContactMethod* cm)
+{
     Q_UNUSED(cm)
 
     if (!QApplication::focusWidget()) {
@@ -262,7 +249,8 @@ CallWidget::onIncomingMessage(::Media::TextRecording* t, ContactMethod* cm) {
 }
 
 void
-CallWidget::setupSmartListMenu() {
+CallWidget::setupSmartListMenu()
+{
     ui->smartList->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(ui->smartList, &QListView::customContextMenuRequested, [=](const QPoint& pos){
         auto idx = ui->smartList->currentIndex();
@@ -320,30 +308,9 @@ CallWidget::setupSmartListMenu() {
     });
 }
 
-void
-CallWidget::findRingAccount(QModelIndex idx1, QModelIndex idx2, QVector<int> vec)
+void CallWidget::setupQRCode(QString ringID)
 {
-    Q_UNUSED(idx1)
-    Q_UNUSED(idx2)
-    Q_UNUSED(vec)
-
-    auto a_count = AccountModel::instance().rowCount();
-    for (int i = 0; i < a_count; ++i) {
-        auto idx = AccountModel::instance().index(i, 0);
-        auto protocol = idx.data(static_cast<int>(Account::Role::Proto));
-        if (static_cast<Account::Protocol>(protocol.toUInt()) == Account::Protocol::RING) {
-            auto username = idx.data(static_cast<int>(Account::Role::Username));
-            ui->ringIdLabel->setText(username.toString());
-            setupQRCode();
-            return;
-        }
-    }
-    ui->ringIdLabel->setText(tr("NO RING ACCOUNT FOUND"));
-}
-
-void CallWidget::setupQRCode()
-{
-    auto rcode = QRcode_encodeString(ui->ringIdLabel->text().toStdString().c_str(),
+    auto rcode = QRcode_encodeString(ringID.toStdString().c_str(),
                                      0, //Let the version be decided by libqrencode
                                      QR_ECLEVEL_L, // Lowest level of error correction
                                      QR_MODE_8, // 8-bit data mode
@@ -381,13 +348,14 @@ void CallWidget::setupQRCode()
 void
 CallWidget::findRingAccount()
 {
-    ui->ringIdLabel->clear();
+    bool ringAccountFound = false;
 
     auto a_count = AccountModel::instance().rowCount();
     for (int i = 0; i < a_count; ++i) {
         auto idx = AccountModel::instance().index(i, 0);
         auto protocol = idx.data(static_cast<int>(Account::Role::Proto));
         if (static_cast<Account::Protocol>(protocol.toUInt()) == Account::Protocol::RING) {
+            ringAccountFound = true;
             auto account = AccountModel::instance().getAccountByModelIndex(idx);
             if (account->displayName().isEmpty())
                 account->displayName() = account->alias();
@@ -395,18 +363,15 @@ CallWidget::findRingAccount()
                 WizardDialog dlg(WizardDialog::MIGRATION, account);
                 dlg.exec();
             }
-            if (ui->ringIdLabel->text().isEmpty()) {
-                auto username = account->username();
-                ui->ringIdLabel->setText(username);
-                setupQRCode();
-            }
         }
     }
-    if (ui->ringIdLabel->text().isEmpty()) {
-        ui->ringIdLabel->setText(tr("NO RING ACCOUNT FOUND"));
+    if (!ringAccountFound) {
         WizardDialog wizardDialog;
         wizardDialog.exec();
     }
+
+    ui->currentAccountWidget->update();
+
 }
 
 void
@@ -414,12 +379,14 @@ CallWidget::callChangedSlot()
 {
     if (not actualCall_)
         return;
+
     ui->callerIdLabel->setText(QString("%1")
                                .arg(actualCall_->formattedName()));
     ui->callerPhoto->setPixmap(
                 QPixmap::fromImage(
                     GlobalInstances::pixmapManipulator()
                     .callPhoto(actualCall_, QSize(130,130)).value<QImage>()));
+    ui->callerBestIdLabel->setText(actualCall_->peerContactMethod()->bestId());
 
     if(actualCall_->state() == Call::State::OVER || actualCall_->state() == Call::State::FAILURE){
         ui->outboundCallLabel->setText(actualCall_->toHumanStateName(Call::State::INITIALIZATION));
@@ -481,50 +448,7 @@ CallWidget::atExit()
 }
 
 void
-CallWidget::on_contactView_doubleClicked(const QModelIndex& index)
-{
-    if (not index.isValid())
-        return;
-
-    auto var = index.child(0,0).data(
-                static_cast<int>(Person::Role::Object));
-    if (var.isValid()) {
-        Person* person = var.value<Person*>();
-        ContactMethod* uri = nullptr;
-        if (person->phoneNumbers().size() == 1) {
-            uri = person->phoneNumbers().at(0);
-        } else if (person->phoneNumbers().size() > 1) {
-            ContactMethodPicker dlg(person->phoneNumbers());
-            auto pos = QCursor::pos();
-            dlg.move(pos.x(), pos.y());
-            if (dlg.exec())
-                uri = dlg.getSelected();
-        }
-        if (uri) {
-            auto outCall = CallModel::instance().dialingCall(uri);
-            outCall->performAction(Call::Action::ACCEPT);
-            ui->mainTabMenu->setCurrentIndex(0);
-            ui->smartList->setFocus();
-        }
-    }
-}
-
-void
-CallWidget::on_historyList_doubleClicked(const QModelIndex& index)
-{
-    if (not index.isValid())
-        return;
-
-    auto number = index.data(static_cast<int>(Call::Role::ContactMethod)).value<ContactMethod*>();
-    if (number) {
-        auto outCall = CallModel::instance().dialingCall(number);
-        outCall->performAction(Call::Action::ACCEPT);
-        ui->mainTabMenu->setCurrentIndex(0);
-        ui->smartList->setFocus();
-    }
-}
-
-void CallWidget::callStateToView(Call* value)
+CallWidget::callStateToView(Call* value)
 {
     if (value) {
         switch (value->state()) {
@@ -538,6 +462,7 @@ void CallWidget::callStateToView(Call* value)
         case Call::State::CONFERENCE:
         case Call::State::HOLD:
             ui->stackedWidget->setCurrentWidget(ui->videoPage);
+            hideMiniSpinner();
             break;
         case Call::State::OVER:
             ui->stackedWidget->setCurrentWidget(ui->welcomePage);
@@ -545,10 +470,13 @@ void CallWidget::callStateToView(Call* value)
         case Call::State::FAILURE:
         case Call::State::ERROR:
             on_cancelButton_clicked();
+            hideMiniSpinner();
             break;
         case Call::State::INITIALIZATION:
         case Call::State::CONNECTED:
         case Call::State::RINGING:
+            miniSpinner_->start();
+            ui->spinnerLabel->show();
             ui->stackedWidget->setCurrentWidget(ui->outboundCallPage);
             break;
         default:
@@ -586,6 +514,14 @@ CallWidget::on_cancelButton_clicked()
     }
 }
 
+void
+CallWidget::on_smartList_clicked(const QModelIndex& index)
+{
+    RecentModel::instance().selectionModel()->setCurrentIndex(
+                RecentModel::instance().peopleProxy()->mapToSource(index),
+                QItemSelectionModel::ClearAndSelect);
+}
+
 void
 CallWidget::on_smartList_doubleClicked(const QModelIndex& index)
 {
@@ -601,7 +537,13 @@ CallWidget::on_smartList_doubleClicked(const QModelIndex& index)
             m = person->phoneNumbers().first();
         }
     }
+
     if (m && !RecentModel::instance().index(0, 0, realIndex).isValid()) {
+
+        QPixmap map = QPixmap::fromImage(
+                        GlobalInstances::pixmapManipulator().callPhoto(m, QSize(130,130)).value<QImage>());
+        ui->callingPhoto->setPixmap(map);
+
         Call* c = CallModel::instance().dialingCall(m);
         c->performAction(Call::Action::ACCEPT);
         setActualCall(c);
@@ -612,23 +554,69 @@ void
 CallWidget::smartListCurrentChanged(const QModelIndex &currentIdx, const QModelIndex &previousIdx)
 {
     Q_UNUSED(previousIdx);
-    if (not currentIdx.isValid())
+    if (not currentIdx.isValid()) {
+        auto widget = ui->stackedWidget->currentWidget();
+        if (widget == ui->messagingPage || widget == ui->videoPage)
+            slidePage(ui->welcomePage);
+        if(actualCall_)
+            setActualCall(nullptr);
         return;
+    }
 
     //catch call of current index
     auto currentIdxCall = RecentModel::instance().getActiveCall(currentIdx);
 
-    if (currentIdxCall && currentIdxCall != actualCall_) { //if it is different from actual call, switch between the two
-        setActualCall(currentIdxCall);
-    } else if (currentIdxCall == nullptr){ // if there is no call attached to this smartlist index (contact tab)
+    if (currentIdxCall) {
+        if (currentIdxCall != actualCall_) //if it is different from actual call, switch between the two
+            setActualCall(currentIdxCall);
+    } else { // if there is no call attached to this smartlist index (contact tab)
         setActualCall(nullptr);
-        showIMOutOfCall(currentIdx); //change page to messaging page with correct behaviour
-    } else { // if non defined behaviour disconnect instant messaging and return to welcome page
+        showIMOutOfCall(currentIdx); // change page to contact request of messaging page with correct behaviour
+    }
+    /*
+    else { // if non defined behaviour disconnect instant messaging and return to welcome page
         setActualCall(nullptr);
         if (imConnection_)
             disconnect(imConnection_);
         ui->stackedWidget->setCurrentWidget(ui->welcomePage);
     }
+    */
+}
+
+void
+CallWidget::configureSendCRPageButton(const QModelIndex& currentIdx)
+{
+   if (currentIdx.isValid()){
+        QVector<ContactMethod*> cmVector = RecentModel::instance().getContactMethods(currentIdx);
+        if (cmVector.size() == 0){
+            ui->sendContactRequestPageButton->hide();
+            return;
+        }
+
+        URI cmUri = cmVector[0]->uri();
+        if (cmUri.protocolHint() == URI::ProtocolHint::RING || uriNeedNameLookup(cmUri)) {
+            ui->sendContactRequestPageButton->show();
+        } else {
+            ui->sendContactRequestPageButton->hide();
+        }
+    } else {
+        ui->sendContactRequestPageButton->hide();
+    }
+}
+
+void
+CallWidget::contactReqListCurrentChanged(const QModelIndex &currentIdx, const QModelIndex &previousIdx)
+{
+    Q_UNUSED(previousIdx)
+
+    if (currentIdx.isValid()) {
+        ui->contactRequestWidget->setCurrentContactRequest(currentIdx);
+        ui->stackedWidget->setCurrentWidget(ui->contactRequestPage);
+    } else {
+        ui->contactRequestWidget->setCurrentContactRequest(QModelIndex());
+        if (ui->stackedWidget->currentWidget() == ui->contactRequestPage)
+            slidePage(ui->welcomePage);
+    }
 }
 
 void
@@ -651,22 +639,10 @@ CallWidget::settingsButtonClicked()
     emit NavigationRequested(ScreenEnum::ConfScreen);
 }
 
-void
-CallWidget::contactButtonClicked(bool checked)
-{
-    ui->mainTabMenu->setCurrentIndex(checked ? 1 : 0);
-}
-
-void
-CallWidget::historicButtonClicked(bool checked)
-{
-    ui->mainTabMenu->setCurrentIndex(checked ? 2 : 0);
-}
-
 void
 CallWidget::searchContactLineEditEntry(const URI &uri)
 {
-    auto cm = PhoneDirectoryModel::instance().getNumber(uri);
+    auto cm = PhoneDirectoryModel::instance().getNumber(uri, getSelectedAccount());
     // if its a new CM, bring it to the top
     if (cm->lastUsed() == 0)
         cm->setLastUsed(QDateTime::currentDateTime().toTime_t());
@@ -675,33 +651,40 @@ CallWidget::searchContactLineEditEntry(const URI &uri)
     RecentModel::instance().selectionModel()->setCurrentIndex(RecentModel::instance().getIndex(cm),
                                                               QItemSelectionModel::ClearAndSelect);
     ui->ringContactLineEdit->clear();
+    cm->setAccount(AvailableAccountModel::instance().currentDefaultAccount(cm));
+}
+
+bool
+CallWidget::uriNeedNameLookup(const URI uri_passed)
+{
+    if (uri_passed.protocolHint() == URI::ProtocolHint::RING_USERNAME ) {
+        return TRUE;
+    } else if (
+        uri_passed.protocolHint() != URI::ProtocolHint::RING && // not a RingID
+        uri_passed.schemeType() == URI::SchemeType::NONE // scheme type not specified
+    ){
+        // if no scheme type has been specified, determine ring vs sip by the current selected account
+        auto idx = AvailableAccountModel::instance().selectionModel()->currentIndex();
+        if (idx.isValid()) {
+            auto account = idx.data((int)Ring::Role::Object).value<Account *>();
+            if (account && account->protocol() == Account::Protocol::RING)
+                return TRUE;
+        }
+    }
+
+    return FALSE;
 }
 
 void
-CallWidget::on_ringContactLineEdit_returnPressed()
+CallWidget::processContactLineEdit()
 {
-    bool lookup_username = false;
     auto contactLineText = ui->ringContactLineEdit->text();
     URI uri_passed = URI(contactLineText);
+    Account* ac = getSelectedAccount();
 
     if (!contactLineText.isNull() && !contactLineText.isEmpty()){
-        if (uri_passed.protocolHint() == URI::ProtocolHint::RING_USERNAME ) {
-            lookup_username = TRUE;
-        } else if (
-            uri_passed.protocolHint() != URI::ProtocolHint::RING && // not a RingID
-            uri_passed.schemeType() == URI::SchemeType::NONE // scheme type not specified
-        ){
-            // if no scheme type has been specified, determine ring vs sip by the first available account
-            auto idx = AvailableAccountModel::instance().index(0, 0);
-            if (idx.isValid()) {
-                auto account = idx.data((int)Ring::Role::Object).value<Account *>();
-                if (account && account->protocol() == Account::Protocol::RING)
-                    lookup_username = TRUE;
-            }
-        }
-
-        if (lookup_username){
-            NameDirectory::instance().lookupName(nullptr, QString(), uri_passed);
+        if (uriNeedNameLookup(uri_passed)){
+            NameDirectory::instance().lookupName(ac, QString(), uri_passed);
         } else {
             searchContactLineEditEntry(uri_passed);
         }
@@ -709,9 +692,15 @@ CallWidget::on_ringContactLineEdit_returnPressed()
 }
 
 void
-CallWidget::on_btnCall_clicked()
+CallWidget::on_ringContactLineEdit_returnPressed()
 {
-    placeCall();
+    processContactLineEdit();
+}
+
+void
+CallWidget::searchBtnClicked()
+{
+    processContactLineEdit();
 }
 
 void
@@ -723,11 +712,65 @@ CallWidget::btnComBarVideoClicked()
     on_smartList_doubleClicked(highLightedIndex_);
 }
 
+void
+CallWidget::selectedAccountChanged(const QModelIndex &current, const QModelIndex &previous)
+{
+    Q_UNUSED(previous)
+
+    if (current.isValid()) {
+        if (ui->selectBar->isHidden()){
+            ui->selectBar->show();
+        }
+        auto ac = current.data(static_cast<int>(Account::Role::Object)).value<Account*>();
+
+        // First, we get back to the welcome view (except if in call)
+
+        if (ui->stackedWidget->currentWidget() != ui->videoPage &&
+            ui->stackedWidget->currentWidget() != ui->welcomePage) {
+            slidePage(ui->welcomePage);
+        }
+
+        // We setup the ringIdLabel and the QRCode
+        auto protocol = ac->protocol();
+        if (protocol == Account::Protocol::RING) {
+            ui->ringIdLabel->setText((ac->registeredName().isEmpty())?ac->username():ac->registeredName());
+            setupQRCode(ac->username());
+        } else {
+            ui->ringIdLabel->setText(tr("NO RING ACCOUNT FOUND"));
+        }
+
+        // Then, we update the pending CR list with those from the newly selected account
+        if (disconnect(crListSelectionConnection_)) {
+            ui->contactRequestList->selectionModel()->clear();
+            // The selection model must be deleted by the application (see QT doc).
+            QItemSelectionModel* sMod = ui->contactRequestList->selectionModel();
+            delete sMod;
+            RecentModel::instance().selectionModel()->clear();
+        }
+
+        ui->contactRequestList->setItemModel(ac->pendingContactRequestModel());
+        crListSelectionConnection_ = connect(ui->contactRequestList->selectionModel(), &QItemSelectionModel::currentChanged,
+                this, &CallWidget::contactReqListCurrentChanged);
+
+        // We modify the currentAccountWidget to reflect the new selected account
+        // if the event wasn't triggered by this widget
+        ui->currentAccountWidget->changeSelectedIndex(current.row());
+
+        if (actualCall_){
+            // keep call on foreground
+            callStateToView(actualCall_);
+        }
+    } else {
+        ui->selectBar->hide();
+        ui->ringIdLabel->setText("");
+    }
+}
+
 void
 CallWidget::showIMOutOfCall(const QModelIndex& nodeIdx)
 {
     ui->contactMethodComboBox->clear();
-
+    configureSendCRPageButton(nodeIdx);
     ui->imNameLabel->setText(QString(tr("Conversation with %1", "%1 is the contact name"))
                              .arg(nodeIdx.data(static_cast<int>(Ring::Role::Name)).toString()));
     auto cmVector = RecentModel::instance().getContactMethods(nodeIdx);
@@ -735,7 +778,7 @@ CallWidget::showIMOutOfCall(const QModelIndex& nodeIdx)
     foreach (const ContactMethod* cm, cmVector) {
         ui->contactMethodComboBox->addItem(cm->uri());
     }
-    slidePage(ui->messagingPage, true);
+    ui->stackedWidget->setCurrentWidget(ui->messagingPage);
     disconnect(imClickedConnection_);
     imClickedConnection_ = connect(ui->listMessageView, &QListView::clicked, [this](const QModelIndex& index) {
         auto urlList = index.data(static_cast<int>(Media::TextRecording::Role::LinkList)).value<QList<QUrl>>();
@@ -817,14 +860,28 @@ CallWidget::slotAccountMessageReceived(const QMap<QString,QString> message,
 void
 CallWidget::on_ringContactLineEdit_textChanged(const QString& text)
 {
-    RecentModel::instance().peopleProxy()->setFilterWildcard(text);
+    RecentModel::instance().peopleProxy()->setFilterRegExp(QRegExp(text, Qt::CaseInsensitive, QRegExp::FixedString));
+}
+
+void
+CallWidget::backToWelcomePage()
+{
+    RecentModel::instance().selectionModel()->clear();
+    disconnect(imConnection_);
+}
+
+void CallWidget::hideMiniSpinner()
+{
+    if(ui->spinnerLabel->isVisible()){
+        miniSpinner_->stop();
+        ui->spinnerLabel->hide();
+    }
 }
 
 void
 CallWidget::on_imBackButton_clicked()
 {
-    RecentModel::instance().selectionModel()->clear();
-    slidePage(ui->welcomePage);
+    backToWelcomePage();
 }
 
 void
@@ -840,14 +897,6 @@ CallWidget::slidePage(QWidget* widget, bool toRight)
     pageAnim_->start();
 }
 
-void
-CallWidget::on_smartList_clicked(const QModelIndex& index)
-{
-    RecentModel::instance().selectionModel()->setCurrentIndex(
-                RecentModel::instance().peopleProxy()->mapToSource(index),
-                QItemSelectionModel::ClearAndSelect);
-}
-
 void
 CallWidget::on_copyCMButton_clicked()
 {
@@ -868,7 +917,7 @@ CallWidget::on_shareButton_clicked()
 }
 
 void
-CallWidget::contactLineEdit_registeredNameFound(const Account* account,NameDirectory::LookupStatus status,
+CallWidget::contactLineEdit_registeredNameFound(Account* account,NameDirectory::LookupStatus status,
                                                 const QString& address,const QString& name)
 {
     URI uri = URI(ui->ringContactLineEdit->text());
@@ -883,8 +932,11 @@ CallWidget::contactLineEdit_registeredNameFound(const Account* account,NameDirec
         case NameDirectory::LookupStatus::SUCCESS:
         {
             uri = URI("ring:" + address);
+            qDebug() << "contactLineEdit username to search: " << username_to_lookup;
             qDebug() << uri;
             searchContactLineEditEntry(uri);
+            auto cm = PhoneDirectoryModel::instance().getNumber(uri);
+            cm->setAccount(account);
             break;
         }
         case NameDirectory::LookupStatus::INVALID_NAME:
@@ -901,3 +953,47 @@ CallWidget::contactLineEdit_registeredNameFound(const Account* account,NameDirec
         }
     }
 }
+
+void
+CallWidget::on_sendContactRequestPageButton_clicked()
+{
+    ui->sendContactRequestWidget->sendCR(RecentModel::instance().selectionModel()->currentIndex());
+        // temporarly changed comportment waiting for message in payload implementation
+        /*ui->sendContactRequestWidget->setup(RecentModel::instance().selectionModel()->currentIndex());
+        slidePage(ui->sendContactRequestPage);*/
+}
+
+void
+CallWidget::on_sendCRBackButton_clicked()
+{
+    ui->stackedWidget->setCurrentWidget(ui->messagingPage);
+}
+
+void
+CallWidget::on_pendingCRBackButton_clicked()
+{
+    ui->contactRequestList->selectionModel()->clear();
+}
+
+Account*
+CallWidget::getSelectedAccount()
+{
+    auto idx = AvailableAccountModel::instance().selectionModel()->currentIndex();
+    if (idx.isValid()) {
+        auto ac = idx.data(static_cast<int>(Ring::Role::Object)).value<Account*>();
+        return ac;
+    }
+    return nullptr;
+}
+
+void CallWidget::on_mainTabMenu_currentChanged(int index)
+{
+    Q_UNUSED(index)
+
+    auto current = ui->mainTabMenu->currentWidget();
+
+    if(current == ui->mainTabMenuPage1)
+        ui->contactRequestList->selectionModel()->clear();
+    else if (current == ui->mainTabMenuPage2)
+        RecentModel::instance().selectionModel()->clear();
+}
