@@ -22,6 +22,7 @@
 #pragma once
 
 #include "threadloop.h"
+#include "noncopyable.h"
 
 #include <gnutls/gnutls.h>
 #include <gnutls/dtls.h>
@@ -39,6 +40,10 @@
 #include <vector>
 #include <map>
 #include <atomic>
+#include <iterator>
+#include <array>
+#include <stdexcept>
+#include <bitset>
 
 namespace ring {
 class IceTransport;
@@ -46,16 +51,21 @@ class IceSocket;
 } // namespace ring
 
 namespace dht { namespace crypto {
-class Certificate;
-class PrivateKey;
+struct Certificate;
+struct PrivateKey;
 }} // namespace dht::crypto
 
 namespace ring { namespace tls {
 
+static constexpr uint8_t MTUS_TO_TEST = 4; //number of mtus to test in path mtu discovery.
+static constexpr int DTLS_MTU {1232}; // (1280 from IPv6 minimum MTU - 40 IPv6 header - 8 UDP header)
+static constexpr uint16_t MIN_MTU {512};
+
 enum class TlsSessionState {
     SETUP,
     COOKIE, // server only
     HANDSHAKE,
+    MTU_DISCOVERY,
     ESTABLISHED,
     SHUTDOWN
 };
@@ -64,9 +74,18 @@ class DhParams {
 public:
     DhParams() = default;
     DhParams(DhParams&&) = default;
+    DhParams(const DhParams& other) {
+        *this = other;
+    }
 
-    /** Take ownership of gnutls_dh_params */
-    explicit DhParams(gnutls_dh_params_t p) : params_(p, gnutls_dh_params_deinit) {};
+    DhParams& operator=(DhParams&& other) = default;
+    DhParams& operator=(const DhParams& other);
+
+    /// \brief Construct by taking ownership of given gnutls DH params
+    ///
+    /// User should not call gnutls_dh_params_deinit on given \a raw_params.
+    /// The object is stolen and its live is manager by our object.
+    explicit DhParams(gnutls_dh_params_t p) : params_ {p, gnutls_dh_params_deinit} {}
 
     /** Deserialize DER or PEM encoded DH-params */
     DhParams(const std::vector<uint8_t>& data);
@@ -88,13 +107,15 @@ public:
     static DhParams generate();
 
 private:
-    std::unique_ptr<gnutls_dh_params_int, decltype(gnutls_dh_params_deinit)&> params_ {nullptr, gnutls_dh_params_deinit};
+    std::unique_ptr<gnutls_dh_params_int, decltype(gnutls_dh_params_deinit)*> params_ {nullptr, gnutls_dh_params_deinit};
 };
 
 struct TlsParams {
     // User CA list for session credentials
     std::string ca_list;
 
+    std::shared_ptr<dht::crypto::Certificate> peer_ca;
+
     // User identity for credential
     std::shared_ptr<dht::crypto::Certificate> cert;
     std::shared_ptr<dht::crypto::PrivateKey> cert_key;
@@ -167,6 +188,8 @@ public:
     ssize_t send(const void* data, std::size_t size);
     ssize_t send(const std::vector<uint8_t>& data);
 
+    uint16_t getMtu();
+
 private:
     using clock = std::chrono::steady_clock;
     using StateHandler = std::function<TlsSessionState(TlsSessionState state)>;
@@ -182,11 +205,12 @@ private:
     TlsSessionState handleStateSetup(TlsSessionState state);
     TlsSessionState handleStateCookie(TlsSessionState state);
     TlsSessionState handleStateHandshake(TlsSessionState state);
+    TlsSessionState handleStateMtuDiscovery(TlsSessionState state);
     TlsSessionState handleStateEstablished(TlsSessionState state);
     TlsSessionState handleStateShutdown(TlsSessionState state);
     std::map<TlsSessionState, StateHandler> fsmHandlers_ {};
     std::atomic<TlsSessionState> state_ {TlsSessionState::SETUP};
-    std::atomic<unsigned int> maxPayload_ {0};
+    std::atomic<unsigned int> maxPayload_;
 
     // IO GnuTLS <-> ICE
     std::mutex txMutex_ {};
@@ -194,12 +218,25 @@ private:
     std::condition_variable rxCv_ {};
     std::list<std::vector<uint8_t>> rxQueue_ {};
 
+    std::mutex reorderBufMutex_;
+    bool flushProcessing_ {false}; ///< protect against recursive call to flushRxQueue
+    std::vector<uint8_t> rawPktBuf_; ///< gnutls incoming packet buffer
+    uint64_t baseSeq_ {0}; ///< sequence number of first application data packet received
+    uint64_t lastRxSeq_ {0}; ///< last received and valid packet sequence number
+    uint64_t gapOffset_ {0}; ///< offset of first byte not received yet
+    clock::time_point lastReadTime_;
+    std::map<uint64_t, std::vector<uint8_t>> reorderBuffer_ {};
+
     ssize_t send_(const uint8_t* tx_data, std::size_t tx_size);
     ssize_t sendRaw(const void*, size_t);
     ssize_t sendRawVec(const giovec_t*, int);
     ssize_t recvRaw(void*, size_t);
     int waitForRawData(unsigned);
 
+    bool initFromRecordState(int offset=0);
+    void handleDataPacket(std::vector<uint8_t>&&, uint64_t);
+    void flushRxQueue();
+
     // Statistics
     std::atomic<std::size_t> stRxRawPacketCnt_ {0};
     std::atomic<std::size_t> stRxRawBytesCnt_ {0};
@@ -231,6 +268,13 @@ private:
     bool setup();
     void process();
     void cleanup();
+
+    // Path mtu discovery
+    std::array<uint16_t, MTUS_TO_TEST>::const_iterator mtuProbe_;
+    unsigned hbPingRecved_ {0};
+    bool pmtudOver_ {false};
+    uint8_t transportOverhead_;
+    void pathMtuHeartbeat();
 };
 
 }} // namespace ring::tls
