@@ -171,9 +171,9 @@ const TypedStateMachine< TypedStateMachine< function , CallPrivate::DaemonState
 /*ERROR          */  {{CP::error      , CP::error     , CP::error     , CP::error        , CP::error        , CP::stop       , CP::error    , CP::error   , CP::nothing}}, /**/
 /*CONF           */  {{CP::nothing    , CP::nothing   , CP::nothing   , CP::warning      , CP::nothing      , CP::stop       , CP::nothing  , CP::stop    , CP::nothing}}, /**/
 /*CONF_HOLD      */  {{CP::nothing    , CP::nothing   , CP::nothing   , CP::warning      , CP::nothing      , CP::stop       , CP::nothing  , CP::stop    , CP::nothing}}, /**/
-/*INIT           */  {{CP::sendProfile, CP::nothing   , CP::warning   , CP::warning      , CP::warning      , CP::stop       , CP::warning  , CP::stop    , CP::nothing}}, /**/
+/*INIT           */  {{CP::nothing    , CP::nothing   , CP::warning   , CP::warning      , CP::warning      , CP::stop       , CP::warning  , CP::stop    , CP::nothing}}, /**/
 /*ABORTED        */  {{CP::error      , CP::error     , CP::error     , CP::error        , CP::error        , CP::error      , CP::error    , CP::error   , CP::nothing}}, /**/
-/*CONNECTED      */  {{CP::sendProfile, CP::nothing   , CP::warning   , CP::warning      , CP::warning      , CP::stop       , CP::warning  , CP::stop    , CP::nothing}}, /**/
+/*CONNECTED      */  {{CP::nothing    , CP::nothing   , CP::warning   , CP::warning      , CP::warning      , CP::stop       , CP::warning  , CP::stop    , CP::nothing}}, /**/
 }};//                                                                                                                                                                        */
 
 //There is no point to have a 2D matrix, only one transition per state is possible
@@ -317,9 +317,6 @@ Call::Call(Call::State startState, const QString& peerName, ContactMethod* numbe
    d_ptr->m_Account          = account;
    d_ptr->m_PeerName         = peerName;
    d_ptr->m_pPeerContactMethod = number;
-   if (number)
-      connect(number, &ContactMethod::unreadTextMessageCountChanged, d_ptr, &CallPrivate::updated);
-   d_ptr->m_pParentCall      = nullptr;
 
    emit changed();
 }
@@ -333,7 +330,6 @@ Call::Call(const QString& confId, const QString& account)
    d_ptr->m_Account      = AccountModel::instance().getById(account.toLatin1());
    d_ptr->m_Type         = (!confId.isEmpty())?Call::Type::CONFERENCE:Call::Type::CALL;
    d_ptr->m_DringId      = confId;
-   d_ptr->m_pParentCall  = nullptr;
 
    setObjectName("Conf:"+confId);
 
@@ -556,9 +552,13 @@ Call* Call::buildHistoryCall(const QMap<QString,QString>& hc)
       //   accId = DRing::Account::ProtocolNames::IP2IP;
    }
 
-   //This corruption has been fixed a while back, but invalid items may still exist
-   if (stopTimeStamp <= 0)
+   // fix invalid time values
+   if (startTimeStamp <= 0) {
+      startTimeStamp = (QDateTime::currentDateTime().currentMSecsSinceEpoch()) / Q_INT64_C(1000);
       stopTimeStamp = startTimeStamp;
+   } else if (stopTimeStamp <= 0) {
+      stopTimeStamp = startTimeStamp;
+   }
 
    //Try to assiciate a contact now, the real contact object is probably not
    //loaded yet, but we can get a placeholder for now
@@ -838,7 +838,7 @@ ContactMethod* Call::peerContactMethod() const
     if (d_ptr->m_pDialNumber)
         return d_ptr->m_pDialNumber;
 
-    return const_cast<ContactMethod*>(ContactMethod::BLANK());
+    return nullptr;
 }
 
 ///Get the peer name
@@ -853,7 +853,7 @@ const QString Call::formattedName() const
     if (type() == Call::Type::CONFERENCE)
         return tr("Conference");
 
-    auto name = peerContactMethod()->roleData(static_cast<int>(Ring::Role::Name)).toString();
+    auto name = peerContactMethod()->bestName();
     if (name.isEmpty())
         return tr("Unknown");
     return name;
@@ -1196,7 +1196,6 @@ void Call::setPeerContactMethod(ContactMethod* cm)
       return;
    }
    d_ptr->m_pPeerContactMethod = cm;
-   connect(cm, &ContactMethod::unreadTextMessageCountChanged, d_ptr, &CallPrivate::updated);
    setDialNumber(cm->uri());
 }
 
@@ -1347,7 +1346,6 @@ Call::State CallPrivate::stateChanged(const QString& newStateName)
    if (q_ptr->lifeCycleState() != Call::LifeCycleState::CREATION && m_pDialNumber) {
       if (!m_pPeerContactMethod) {
           m_pPeerContactMethod = PhoneDirectoryModel::instance().fromTemporary(m_pDialNumber);
-          connect(m_pPeerContactMethod, &ContactMethod::unreadTextMessageCountChanged, this, &CallPrivate::updated);
       }
       m_pDialNumber->deleteLater();
       m_pDialNumber = nullptr;
@@ -1780,19 +1778,36 @@ void CallPrivate::call()
 
     // Warning: m_pDialNumber can become nullptr when linking directly
     URI uri {peerCM->uri()};
+
+    // Better late than never. Set the scheme.
+    if (m_Account && m_Account->protocol() == Account::Protocol::RING &&
+      uri.schemeType() == URI::SchemeType::NONE &&
+      uri.protocolHint() != URI::ProtocolHint::RING) {
+       uri.setSchemeType(URI::SchemeType::RING);
+    }
+
     if (!m_pPeerContactMethod) {
         m_pPeerContactMethod = PhoneDirectoryModel::instance().getNumber(uri, q_ptr->account());
-        connect(m_pPeerContactMethod, &ContactMethod::unreadTextMessageCountChanged, this, &CallPrivate::updated);
     }
 
     // m_pDialNumber is now discarded
     m_pDialNumber->deleteLater();
     m_pDialNumber = nullptr;
 
+    //set the start timestamp to now, even if the call fails immediately, so that it has a timestamp
+    //this behaviour is the same as if the call intialized in the daemon but then failed shortly after
+    setStartTimeStamp();
+
     //Refresh peerCM
     peerCM = q_ptr->peerContactMethod();
     peerCM->addCall(q_ptr);
 
+    // dring can print "No suitable account to create outgoing call" if the
+    // URI isn't properly formatted (like sip:foobar).
+    if (m_Account && m_Account->protocol() == Account::Protocol::RING && uri.protocolHint() != URI::ProtocolHint::RING) {
+       qWarning() << "The URI isn't a RingId. It is possible the call wont work" << uri.full();
+    }
+
     m_DringId = CallManager::instance().placeCall(m_Account->id(), uri.full());
 
     // This can happen when the daemon cannot allocate memory
@@ -1804,7 +1819,6 @@ void CallPrivate::call()
     }
     setObjectName("Call:"+m_DringId);
 
-    setStartTimeStamp();
     CallModel::instance().registerCall(q_ptr);
 
     connect(peerCM, SIGNAL(presentChanged(bool)), this, SLOT(updated()));
@@ -1899,13 +1913,15 @@ void CallPrivate::start()
    if (m_pDialNumber) {
       if (!m_pPeerContactMethod) {
           m_pPeerContactMethod = PhoneDirectoryModel::instance().fromTemporary(m_pDialNumber);
-          connect(m_pPeerContactMethod, &ContactMethod::unreadTextMessageCountChanged, this, &CallPrivate::updated);
       }
       m_pDialNumber->deleteLater();
       m_pDialNumber = nullptr;
    }
    setStartTimeStamp();
    initTimer();
+
+   /* we want to exchange vCards once the call starts so both peers have updated profiles */
+   sendProfile();
 }
 
 ///Toggle the timer
@@ -1932,6 +1948,9 @@ void CallPrivate::startWeird()
    qDebug() << "Starting call. callId : " << q_ptr  << "ConfId:" << q_ptr;
    setStartTimeStamp();
    qDebug() << "Warning : call " << q_ptr << " had an unexpected transition of state at its start.";
+
+   /* we want to exchange vCards once the call starts so both peers have updated profiles */
+   sendProfile();
 }
 
 ///Print a warning
@@ -2170,6 +2189,8 @@ QVariant Call::roleData(int role) const
          return dialNumber();
       case static_cast<int>(Ring::Role::Number):
       case static_cast<int>(Call::Role::Number):
+         return peerContactMethod()->bestId();
+      case static_cast<int>(Ring::Role::URI):
          return peerContactMethod()->uri();
       case Qt::DecorationRole:
          return GlobalInstances::pixmapManipulator().decorationRole(this);
@@ -2232,6 +2253,7 @@ QVariant Call::roleData(int role) const
       case static_cast<int>(Call::Role::StopTime):
          return (int) d_ptr->m_pStopTimeStamp;
       case static_cast<int>(Call::Role::IsPresent):
+      case static_cast<int>(Ring::Role::IsPresent):
          return peerContactMethod()->isPresent();
       case static_cast<int>(Call::Role::IsTracked):
          return peerContactMethod()->isTracked();
